@@ -1,62 +1,64 @@
 #!/usr/bin/perl
 
+use strict;
+use warnings;
+use Dir::Split qw(split_dir);
+
+our(%form, %form_o);
+my $return = -255;
+
+
+
 #
-# Uncomment the lines at the bottom accordingly
-# whether numeric or characteristic splitting shall be
+# Modify following lines accordingly to whether 
+# numeric or characteristic splitting shall be
 # committed.
 #
 
-use strict 'vars';
-use warnings;
+my %num_options = (  
+   mode    =>    'num',
 
-use Dir::Split qw(split_dir);
-
-our($return, %num_options, %char_options, %form, %form_o);
-$return = -255;
-
-
-%num_options = (  mode    =>    'num',
-
-                  source  =>    '/source',
-		  target  =>    '/target',
-
-                  options => {  verbose     =>           1,
-                                override    =>           0,
-                  },
-                  sub_dir => {  identifier  =>       'sub',
-                                file_limit  =>           2,
-                                file_sort   =>         '+',
-                  },
-                  suffix  => {  separator   =>         '-',
-                                continue    =>           1,
-                                length      =>           5,
-                  },
+   source  =>    '/source',
+   target  =>    '/target',
+ 
+   options => {  verbose     =>           1,
+                 override    =>           0,
+   },
+   sub_dir => {  identifier  =>       'sub',
+                 file_limit  =>           2,
+                 file_sort   =>         '+',
+   },
+   suffix  => {  separator   =>         '-',
+                 continue    =>           1,
+                 length      =>           5,
+   },
 );
 
-%char_options = (  mode    =>    'char',
+my %char_options = (  
+   mode    =>    'char',
 
-                   source  =>    '/source',
-		   target  =>    '/target',
-
-                   options => {  verbose     =>           1,
-                                 override    =>           0,
-                   },
-                   sub_dir => {  identifier  =>       'sub',
-                   },
-                   suffix  => {  separator   =>         '-',
-                                 case        =>     'upper',
-                                 length      =>           1,
-                   },
+   source  =>    '/source',
+   target  =>    '/target',
 
+   options => {  verbose     =>           1,
+                 override    =>           0,
+   },
+   sub_dir => {  identifier  =>       'sub',
+   },
+   suffix  => {  separator   =>         '-',
+                 case        =>     'upper',
+                 length      =>           1,
+   },
 );
 
 
-# traversal mode and options.
+# traversal mode and options
 #
 #$Dir::Split::Traverse        = 1;
 #$Dir::Split::Traverse_unlink = 1;
 #$Dir::Split::Traverse_rmdir  = 1;
 
+
 # numeric splitting
 #
 #$return = split_dir(%num_options);
@@ -65,6 +67,11 @@ $return = -255;
 #
 #$return = split_dir(%char_options);
 
+
+
+# END OF CONFIG
+###############
+
 # action
 if ($return == 1) { formwrite('track') }
 # no action
@@ -104,6 +111,7 @@ elsif ($return == -2) {
     
     $form_o{header} = 'FAILURE';
     $form_o{ul} = '-' x length($form_o{header});
+    
     formwrite('start_debug');
     
     for (@{$Dir::Split::failure{copy}}) {
